@@ -489,6 +489,12 @@ MessageTray.prototype = {
         }
     },
 
+    removeSourceByApp: function(app) {
+        for (let source in this._sources)
+            if (this._sources[source].app == app)
+                this.removeSource(this._sources[source]);
+    },
+
     removeNotification: function(notification) {
         if (this._notification == notification && (this._notificationState == State.SHOWN || this._notificationState == State.SHOWING)) {
             if (this._notificationTimeoutId) {
@@ -690,7 +696,6 @@ MessageTray.prototype = {
     },
 
     _hideNotification: function() {
-        this._notificationRemoved = false;
         this._notification.popIn();
 
         this._tween(this._notificationBin, "_notificationState", State.HIDDEN,
@@ -704,6 +709,7 @@ MessageTray.prototype = {
     },
 
     _hideNotificationCompleted: function() {
+        this._notificationRemoved = false;
         this._notificationBin.hide();
         this._notificationBin.child = null;
         this._notification = null;
