@@ -89,8 +89,7 @@ InfoBar.prototype = {
                                   x_fill: true,
                                   y_fill: false });
         this._label = new St.Label();
-        this._undo = new St.Button({ label: _("Undo"),
-                                     style_class: 'info-bar-link-button' });
+        this._undo = new St.Button({ style_class: 'info-bar-link-button' });
 
         let bin = new St.Bin({ x_fill: false,
                                y_fill: false,
@@ -153,7 +152,7 @@ InfoBar.prototype = {
             this._overviewWasHidden = true;
     },
 
-    setMessage: function(text, undoCallback) {
+    setMessage: function(text, undoCallback, undoLabel) {
         if (this._timeoutId)
             Mainloop.source_remove(this._timeoutId);
 
@@ -177,6 +176,11 @@ InfoBar.prototype = {
 
         this._timeoutId = Mainloop.timeout_add_seconds(INFO_BAR_HIDE_TIMEOUT, Lang.bind(this, this._onTimeout));
 
+        if (undoLabel)
+            this._undo.label = undoLabel;
+        else
+            this._undo.label = _("Undo");
+
         this._undoCallback = undoCallback;
         if (undoCallback)
             this._undo.show();
@@ -197,8 +201,7 @@ Overview.prototype = {
         this.infoBar = new InfoBar();
         this._group.add_actor(this.infoBar.actor);
 
-        this._workspacesViewSwitch = new WorkspacesView.WorkspacesViewSwitch();
-        this._workspacesViewSwitch.connect('view-changed', Lang.bind(this, this._onViewChanged));
+        this._workspacesManager = null;
 
         this.visible = false;
         this.animationInProgress = false;
@@ -251,43 +254,16 @@ Overview.prototype = {
         this._workspaces = null;
     },
 
-    _createControlsBar: function() {
-        this._workspacesBar = new St.BoxLayout({ 'pack-start': true,
-                                                 style_class: 'workspaces-bar' });
-        this._workspacesBar.move_by(this._workspacesBarX, this._workspacesBarY);
-
-        let controlsBar = this._workspacesViewSwitch.createControlsBar();
-        let bar = this._workspaces.createControllerBar();
-        this._workspacesBar.add(bar, { expand: true, 'x-fill': true, 'y-fill': true,
-                                       y_align: St.Align.MIDDLE, x_align: St.Align.START });
-        this._workspacesBar.add(controlsBar, {x_align: St.Align.END});
-        this._workspacesBar.width = this._workspacesBarWidth;
-
-        this._group.add_actor(this._workspacesBar);
-        this._workspacesBar.raise(this._workspaces.actor);
-    },
-
     _onViewChanged: function() {
         if (!this.visible)
             return;
-        //Remove old worspacesView
-        this._group.remove_actor(this._workspacesBar);
-        this._workspaces.hide();
-        this._group.remove_actor(this._workspaces.actor);
-        this._workspaces.destroy();
-        this._workspacesBar.destroy();
 
-        this._workspaces = this._workspacesViewSwitch.createCurrentWorkspaceView(this._workspacesWidth, this._workspacesHeight,
-                                                                             this._workspacesX, this._workspacesY, false);
+        this._workspaces = this._workspacesManager.workspacesView;
 
-        //Show new workspacesView
+        // Show new workspacesView
         this._group.add_actor(this._workspaces.actor);
+        this._workspacesBar.raise(this._workspaces.actor);
         this._dash.actor.raise(this._workspaces.actor);
-
-        this._createControlsBar();
-
-        // Set new position and scale to workspaces.
-        this.emit('showing');
     },
 
     _recalculateGridSizes: function () {
@@ -308,6 +284,7 @@ Overview.prototype = {
 
     relayout: function () {
         let primary = global.get_primary_monitor();
+        let rtl = (St.Widget.get_default_direction () == St.TextDirection.RTL);
 
         this._recalculateGridSizes();
 
@@ -329,10 +306,19 @@ Overview.prototype = {
         this._workspacesHeight = Math.floor(displayGridRowHeight * workspaceRowsUsed
                                    - WORKSPACE_GRID_PADDING * (primary.height / primary.width) * 2);
 
-        this._workspacesX = displayGridColumnWidth + WORKSPACE_GRID_PADDING;
+        if (rtl) {
+            this._workspacesX = WORKSPACE_GRID_PADDING;
+        } else {
+            this._workspacesX = displayGridColumnWidth + WORKSPACE_GRID_PADDING;
+        }
         this._workspacesY = Math.floor(displayGridRowHeight + WORKSPACE_GRID_PADDING * (primary.height / primary.width));
 
-        this._dash.actor.set_position(0, contentY);
+        if (rtl) {
+            this._dash.actor.set_position(primary.width - displayGridColumnWidth, contentY);
+        } else {
+            this._dash.actor.set_position(0, contentY);
+        }
+
         this._dash.actor.set_size(displayGridColumnWidth, contentHeight);
         this._dash.searchArea.height = this._workspacesY - contentY;
         this._dash.sectionArea.height = this._workspacesHeight;
@@ -344,7 +330,7 @@ Overview.prototype = {
 
         // place the 'Add Workspace' button in the bottom row of the grid
         this._workspacesBarX = this._workspacesX;
-        this._workspacesBarWidth = primary.width - this._workspacesBarX - WORKSPACE_GRID_PADDING;
+        this._workspacesBarWidth = this._workspacesWidth;
         this._workspacesBarY = primary.height - displayGridRowHeight;
 
         // The parent (this._group) is positioned at the top left of the primary monitor
@@ -356,15 +342,22 @@ Overview.prototype = {
                                          this._workspacesY);
         // Dynamic width
         this._paneContainer.height = this._workspacesHeight;
+        if (rtl) {
+            this._paneContainer.connect('notify::width', Lang.bind(this, function (paneContainer) {
+                paneContainer.x = this._dash.actor.x - (DEFAULT_PADDING + paneContainer.width);
+            }));
+        }
 
-        this._transparentBackground.set_position(this._paneContainer.x, this._paneContainer.y);
-        this._transparentBackground.set_size(primary.width - this._paneContainer.x,
-                                             this._paneContainer.height);
+        this._transparentBackground.set_position(primary.x, primary.y);
+        this._transparentBackground.set_size(primary.width, primary.height);
 
     },
 
-    addPane: function (pane) {
-        this._paneContainer.add(pane.actor, { expand: true, y_fill: false, y_align: St.Align.START });
+    addPane: function (pane, align) {
+        pane.actor.height = .9 * this._workspacesHeight;
+        this._paneContainer.add(pane.actor, { expand: true,
+                                              y_fill: false,
+                                              y_align: align });
         // When a pane is displayed, we raise the transparent background to the top
         // and connect to button-release-event on it, then raise the pane above that.
         // The idea here is that clicking anywhere outside the pane should close it.
@@ -376,10 +369,17 @@ Overview.prototype = {
                 this._transparentBackground.raise_top();
                 this._paneContainer.raise_top();
                 this._paneContainer.show();
+                this._paneReady = false;
                 if (backgroundEventId != null)
                     this._transparentBackground.disconnect(backgroundEventId);
-                backgroundEventId = this._transparentBackground.connect('button-release-event', Lang.bind(this, function () {
-                    this._activeDisplayPane.close();
+                backgroundEventId = this._transparentBackground.connect('captured-event', Lang.bind(this, function (actor, event) {
+                    if (event.get_source() != this._transparentBackground)
+                        return false;
+                    if (event.type() == Clutter.EventType.BUTTON_PRESS)
+                        this._paneReady = true;
+                    if (event.type() == Clutter.EventType.BUTTON_RELEASE
+                        && this._paneReady)
+                        this._activeDisplayPane.close();
                     return true;
                 }));
                 this._workspaces.actor.opacity = 64;
@@ -451,8 +451,14 @@ Overview.prototype = {
         this._dash.show();
 
         /* TODO: make this stuff dynamic */
-        this._workspaces = this._workspacesViewSwitch.createCurrentWorkspaceView(this._workspacesWidth, this._workspacesHeight,
-                                                                             this._workspacesX, this._workspacesY, true);
+        this._workspacesManager =
+            new WorkspacesView.WorkspacesManager(this._workspacesWidth,
+                                                 this._workspacesHeight,
+                                                 this._workspacesX,
+                                                 this._workspacesY);
+        this._workspacesManager.connect('view-changed',
+                                        Lang.bind(this, this._onViewChanged));
+        this._workspaces = this._workspacesManager.workspacesView;
         this._group.add_actor(this._workspaces.actor);
 
         // The workspaces actor is as big as the screen, so we have to raise the dash above it
@@ -460,7 +466,13 @@ Overview.prototype = {
         // be as big as the screen.
         this._dash.actor.raise(this._workspaces.actor);
 
-        this._createControlsBar();
+        this._workspacesBar = this._workspacesManager.controlsBar.actor;
+        this._workspacesBar.set_position(this._workspacesBarX,
+                                         this._workspacesBarY);
+        this._workspacesBar.width = this._workspacesBarWidth;
+
+        this._group.add_actor(this._workspacesBar);
+        this._workspacesBar.raise(this._workspaces.actor);
 
         // All the the actors in the window group are completely obscured,
         // hiding the group holding them while the Overview is displayed greatly
@@ -490,7 +502,7 @@ Overview.prototype = {
                            onCompleteScope: this
                           });
 
-        // Make Dash fade in so that it doesn't appear to big.
+        // Make Dash fade in so that it doesn't appear too big.
         this._dash.actor.opacity = 0;
         Tweener.addTween(this._dash.actor,
                          { opacity: 255,
@@ -512,9 +524,6 @@ Overview.prototype = {
             this._activeDisplayPane.close();
         this._workspaces.hide();
 
-        this._workspacesBar.destroy();
-        this._workspacesBar = null;
-
         // Create a zoom in effect by transforming the Overview group so that
         // the active workspace fills up the whole screen. The opposite
         // transition is used in show().
@@ -579,6 +588,11 @@ Overview.prototype = {
         this._workspaces.destroy();
         this._workspaces = null;
 
+        this._workspacesBar.destroy();
+        this._workspacesBar = null;
+
+        this._workspacesManager = null;
+
         this._dash.hide();
         this._group.hide();
 
