@@ -39,7 +39,7 @@ const TRAY_BORDER_WIDTH = 0;
 
 const HOT_CORNER_ACTIVATION_TIMEOUT = 0.5;
 
-const STANDARD_TRAY_ICON_ORDER = ['keyboard', 'volume', 'bluetooth', 'network', 'battery']
+const STANDARD_TRAY_ICON_ORDER = ['keyboard', 'volume', 'bluetooth', 'network', 'battery'];
 const STANDARD_TRAY_ICON_IMPLEMENTATIONS = {
     'bluetooth-applet': 'bluetooth',
     'gnome-volume-control-applet': 'volume',
@@ -130,7 +130,7 @@ TextShadower.prototype = {
             child.allocate(childBox, flags);
         }
     }
-}
+};
 
 /**
  * AppPanelMenu:
@@ -152,15 +152,17 @@ AppPanelMenu.prototype = {
         this._activeSequence = null;
         this._startupSequences = {};
 
-        this.actor = new St.BoxLayout({ name: 'appMenu' });
-        this._iconBox = new Shell.Slicer({ name: 'appMenuIcon' });
-        this.actor.add(this._iconBox);
-        this._label = new TextShadower();
-        this.actor.add(this._label.actor, { expand: true, y_fill: true });
-        this.actor.connect('notify::allocation', Lang.bind(this, this._repositionLabel));
+        this.actor = new St.Bin({ name: 'appMenu' });
+        this._container = new Shell.GenericContainer();
+        this.actor.set_child(this._container);
+        this._container.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
+        this._container.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
+        this._container.connect('allocate', Lang.bind(this, this._allocate));
 
-        this._startupBox = new St.BoxLayout();
-        this.actor.add(this._startupBox);
+        this._iconBox = new Shell.Slicer({ name: 'appMenuIcon' });
+        this._container.add_actor(this._iconBox);
+        this._label = new TextShadower();
+        this._container.add_actor(this._label.actor);
 
         Main.overview.connect('hiding', Lang.bind(this, function () {
             this.actor.opacity = 255;
@@ -181,14 +183,63 @@ AppPanelMenu.prototype = {
         this._sync();
     },
 
-    _repositionLabel: function() {
-        this._label.actor.x = Math.floor(AppDisplay.APPICON_SIZE / 2);
-        let actorAlloc = this.actor.allocation;
-        let actorHeight = actorAlloc.y2 - actorAlloc.y1;
-        let labelAlloc = this._label.actor.allocation;
-        let labelHeight = labelAlloc.y2 - labelAlloc.y1;
-        this._label.actor.y = Math.floor((actorHeight - labelHeight) / 2);
-        this._label.actor.fixed_position_set = true;
+    _getPreferredWidth: function(actor, forHeight, alloc) {
+        let [minSize, naturalSize] = this._iconBox.get_preferred_width(forHeight);
+        alloc.min_size = minSize;
+        alloc.natural_size = naturalSize;
+        [minSize, naturalSize] = this._label.actor.get_preferred_width(forHeight);
+        alloc.min_size = alloc.min_size + Math.max(0, minSize - Math.floor(alloc.min_size / 2));
+        alloc.natural_size = alloc.natural_size + Math.max(0, naturalSize - Math.floor(alloc.natural_size / 2));
+    },
+
+    _getPreferredHeight: function(actor, forWidth, alloc) {
+        let [minSize, naturalSize] = this._iconBox.get_preferred_height(forWidth);
+        alloc.min_size = minSize;
+        alloc.natural_size = naturalSize;
+        [minSize, naturalSize] = this._label.actor.get_preferred_height(forWidth);
+        if (minSize > alloc.min_size)
+            alloc.min_size = minSize;
+        if (naturalSize > alloc.natural_size)
+            alloc.natural_size = naturalSize;
+    },
+
+    _allocate: function(actor, box, flags) {
+        let allocWidth = box.x2 - box.x1;
+        let allocHeight = box.y2 - box.y1;
+        let childBox = new Clutter.ActorBox();
+
+        let [minWidth, minHeight, naturalWidth, naturalHeight] = this._iconBox.get_preferred_size();
+
+        let direction = this.actor.get_direction();
+
+        let yPadding = Math.floor(Math.max(0, allocHeight - naturalHeight) / 2);
+        childBox.y1 = yPadding;
+        childBox.y2 = childBox.y1 + Math.min(naturalHeight, allocHeight);
+        if (direction == St.TextDirection.LTR) {
+            childBox.x1 = 0;
+            childBox.x2 = childBox.x1 + Math.min(naturalWidth, allocWidth);
+        } else {
+            childBox.x1 = Math.max(0, allocWidth - naturalWidth);
+            childBox.x2 = allocWidth;
+        }
+        this._iconBox.allocate(childBox, flags);
+
+        let iconWidth = childBox.x2 - childBox.x1;
+
+        [minWidth, minHeight, naturalWidth, naturalHeight] = this._label.actor.get_preferred_size();
+
+        yPadding = Math.floor(Math.max(0, allocHeight - naturalHeight) / 2);
+        childBox.y1 = yPadding;
+        childBox.y2 = childBox.y1 + Math.min(naturalHeight, allocHeight);
+
+        if (direction == St.TextDirection.LTR) {
+            childBox.x1 = Math.floor(iconWidth / 2);
+            childBox.x2 = Math.min(childBox.x1 + naturalWidth, allocWidth);
+        } else {
+            childBox.x2 = allocWidth - Math.floor(iconWidth / 2);
+            childBox.x1 = Math.max(0, childBox.x2 - naturalWidth);
+        }
+        this._label.actor.allocate(childBox, flags);
     },
 
     _sync: function() {
@@ -220,7 +271,7 @@ AppPanelMenu.prototype = {
         this._label.setText('');
         let icon;
         if (this._focusedApp != null) {
-            icon = this._focusedApp.create_icon_texture(AppDisplay.APPICON_SIZE);
+            icon = this._focusedApp.get_faded_icon(AppDisplay.APPICON_SIZE);
             this._label.setText(this._focusedApp.get_name());
         } else if (this._activeSequence != null) {
             icon = this._activeSequence.create_icon(AppDisplay.APPICON_SIZE);
@@ -228,17 +279,15 @@ AppPanelMenu.prototype = {
         } else {
             icon = null;
         }
-        
+
         if (icon != null) {
-            let faded = Shell.fade_app_icon(icon); /* TODO consider caching */
-            this._iconBox.set_child(faded);
+            this._iconBox.set_child(icon);
             this._iconBox.show();
         }
-        this._repositionLabel();
 
         this.emit('changed');
     }
-}
+};
 
 Signals.addSignalMethods(AppPanelMenu.prototype);
 
@@ -553,15 +602,22 @@ Panel.prototype = {
             displayDate.setMinutes(displayDate.getMinutes() + 1);
             msecRemaining += 60000;
         }
-	/* If there is no am or pm, time format is 24h */
-	let isTime24h = displayDate.toLocaleFormat("x%p") == "x";
-	if (isTime24h) {
-	  /* Translators: This is the time format used in 24-hour mode. */
-	  this._clock.set_text(displayDate.toLocaleFormat(_("%a %R")));
-	} else {
-	  /* Translators: This is a time format used for AM/PM. */
-	  this._clock.set_text(displayDate.toLocaleFormat(_("%a %l:%M %p")));
-	}
+
+        // if the locale representations of 05:00 and 17:00 do not
+        // start with the same 2 digits, it must be a 24h clock
+        let fiveAm = new Date();
+        fiveAm.setHours(5);
+        let fivePm = new Date();
+        fivePm.setHours(17);
+        let isTime24h = fiveAm.toLocaleFormat("%X").substr(0,2) !=
+                        fivePm.toLocaleFormat("%X").substr(0,2);
+        if (isTime24h) {
+            /* Translators: This is the time format used in 24-hour mode. */
+            this._clock.set_text(displayDate.toLocaleFormat(_("%a %R")));
+        } else {
+            /* Translators: This is a time format used for AM/PM. */
+            this._clock.set_text(displayDate.toLocaleFormat(_("%a %l:%M %p")));
+        }
         Mainloop.timeout_add(msecRemaining, Lang.bind(this, this._updateClock));
         return false;
     },
