@@ -59,7 +59,7 @@ PlaceInfo.prototype = {
     isRemovable: function() {
         return false;
     }
-}
+};
 
 function PlaceDeviceInfo(mount) {
     this._init(mount);
@@ -77,7 +77,7 @@ PlaceDeviceInfo.prototype = {
 
     iconFactory: function(size) {
         let icon = this._mount.get_icon();
-        return Shell.TextureCache.get_default().load_gicon(icon, size);
+        return St.TextureCache.get_default().load_gicon(icon, size);
     },
 
     launch: function() {
@@ -93,13 +93,20 @@ PlaceDeviceInfo.prototype = {
         if (!this.isRemovable())
             return;
 
-        this._mount.unmount(0, null, Lang.bind(this, this._removeFinish), null);
+        this._mount.unmount(0, null, Lang.bind(this, this._removeFinish));
     },
 
     _removeFinish: function(o, res, data) {
-        this._mount.unmount_finish(res);
+        try {
+            this._mount.unmount_finish(res);
+        } catch (e) {
+            let message = _("Failed to unmount '%s'").format(o.get_name());
+            Main.overview.infoBar.setMessage(message,
+                                             Lang.bind(this, this.remove),
+                                             _("Retry"));
+        }
     }
-}
+};
 
 
 function PlacesManager() {
@@ -122,7 +129,7 @@ PlacesManager.prototype = {
         let homeIcon = Shell.util_get_icon_for_uri (homeUri);
         this._home = new PlaceInfo('special:home', homeLabel,
             function(size) {
-                return Shell.TextureCache.get_default().load_gicon(homeIcon, size);
+                return St.TextureCache.get_default().load_gicon(homeIcon, size);
             },
             function() {
                 Gio.app_info_launch_default_for_uri(homeUri, global.create_app_launch_context());
@@ -135,7 +142,7 @@ PlacesManager.prototype = {
         let desktopIcon = Shell.util_get_icon_for_uri (desktopUri);
         this._desktopMenu = new PlaceInfo('special:desktop', desktopLabel,
             function(size) {
-                return Shell.TextureCache.get_default().load_gicon(desktopIcon, size);
+                return St.TextureCache.get_default().load_gicon(desktopIcon, size);
             },
             function() {
                 Gio.app_info_launch_default_for_uri(desktopUri, global.create_app_launch_context());
@@ -143,7 +150,7 @@ PlacesManager.prototype = {
 
         this._connect = new PlaceInfo('special:connect', _("Connect to..."),
             function (size) {
-                return Shell.TextureCache.get_default().load_icon_name("applications-internet", size);
+                return St.TextureCache.get_default().load_icon_name("applications-internet", size);
             },
             function () {
                 new Shell.Process({ args: ['nautilus-connect-server'] }).run();
@@ -310,7 +317,7 @@ PlacesManager.prototype = {
 
             let item = new PlaceInfo('bookmark:' + bookmark, label,
                 function(size) {
-                    return Shell.TextureCache.get_default().load_gicon(icon, size);
+                    return St.TextureCache.get_default().load_gicon(icon, size);
                 },
                 function() {
                     Gio.app_info_launch_default_for_uri(bookmark, global.create_app_launch_context());
@@ -401,31 +408,45 @@ DashPlaceDisplayItem.prototype = {
         this.name = info.name;
         this._info = info;
         this._icon = info.iconFactory(PLACES_ICON_SIZE);
-        this.actor = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL,
-                                   spacing: 4 });
-        let text = new St.Button({ style_class: 'places-item',
-                                   label: info.name,
-                                   x_align: St.Align.START });
-        text.connect('clicked', Lang.bind(this, this._onClicked));
-        let iconBox = new St.Bin({ child: this._icon, reactive: true });
-        iconBox.connect('button-release-event',
-                        Lang.bind(this, this._onClicked));
-        this.actor.append(iconBox, Big.BoxPackFlags.NONE);
-        this.actor.append(text, Big.BoxPackFlags.EXPAND);
+
+        this.actor = new St.Clickable({ style_class: 'places-item',
+                                        reactive: true,
+                                        x_align: St.Align.START,
+                                        x_fill: true });
+
+        let box = new St.BoxLayout({ style_class: 'places-item-box' });
+        this.actor.set_child(box);
+
+        let bin = new St.Bin({ child: this._icon });
+        box.add(bin);
+
+        let text = new St.Label({ text: info.name });
+        box.add(text, { expand: true, x_fill: true });
 
         if (info.isRemovable()) {
-            let removeIcon = Shell.TextureCache.get_default().load_icon_name ('media-eject', PLACES_ICON_SIZE);
-            let removeIconBox = new St.Button({ child: removeIcon,
-                                                reactive: true });
-            this.actor.append(removeIconBox, Big.BoxPackFlags.NONE);
+            let removeIcon = St.TextureCache.get_default().load_icon_name ('media-eject', PLACES_ICON_SIZE);
+            let removeIconBox = new St.Clickable({ child: removeIcon,
+                                                   reactive: true });
+            box.add(removeIconBox);
             removeIconBox.connect('clicked',
                                   Lang.bind(this, function() {
                                                   this._info.remove();
                                             }));
         }
 
+        this.actor.connect('clicked', Lang.bind(this, this._onClicked));
+        this.actor.connect('notify::hover',
+                           Lang.bind(this, this._onHoverChanged));
+        this.actor.connect('button-press-event',
+                           Lang.bind(this, this._onButtonPress));
+        this.actor.connect('button-release-event',
+                           Lang.bind(this, this._onButtonRelease));
+
         this.actor._delegate = this;
-        let draggable = DND.makeDraggable(this.actor);
+        this._dragStartX = null;
+        this._dragStartY = null;
+        this._draggable = DND.makeDraggable(this.actor,
+                                            { manualMode: true });
     },
 
     _onClicked: function(b) {
@@ -433,6 +454,36 @@ DashPlaceDisplayItem.prototype = {
         Main.overview.hide();
     },
 
+    _onButtonPress: function(actor, event) {
+        if (event.get_button() != 1)
+            return false;
+
+        let [stageX, stageY] = event.get_coords();
+        this._dragStartX = stageX;
+        this._dragStartY = stageY;
+        return false;
+    },
+
+    _onButtonRelease: function(actor, event) {
+        if (event.get_button() != 1)
+            return false;
+
+        this._dragStartX = null;
+        this._dragStartY = null;
+        return false;
+    },
+
+    _onHoverChanged: function(button) {
+        let hover = button.hover;
+        if (!hover) {
+            if (button.pressed && this._dragStartX != null) {
+                button.fake_release();
+                this._draggable.startDrag(this._dragStartX, this._dragStartY,
+                                          global.get_current_time());
+            }
+        }
+    },
+
     getDragActorSource: function() {
         return this._icon;
     },
@@ -581,4 +632,4 @@ PlaceSearchProvider.prototype = {
         let places = previousResults.map(function (id) { return Main.placesManager.lookupPlaceById(id); });
         return this._searchPlaces(places, terms);
     }
-}
+};
