@@ -4,15 +4,10 @@
 #define __SHELL_DRAWING_H__
 
 #include <clutter/clutter.h>
+#include "st.h"
 
 G_BEGIN_DECLS
 
-ClutterCairoTexture *shell_create_vertical_gradient (ClutterColor *top,
-                                                     ClutterColor *bottom);
-
-ClutterCairoTexture *shell_create_horizontal_gradient (ClutterColor *left,
-                                                       ClutterColor *right);
-
 typedef enum {
   SHELL_POINTER_UP,
   SHELL_POINTER_DOWN,
@@ -20,12 +15,12 @@ typedef enum {
   SHELL_POINTER_RIGHT
 } ShellPointerDirection;
 
-void shell_draw_box_pointer (ClutterCairoTexture   *texture,
+void shell_draw_box_pointer (StDrawingArea         *area,
                              ShellPointerDirection  direction,
                              ClutterColor          *border_color,
                              ClutterColor          *background_color);
 
-void shell_draw_clock (ClutterCairoTexture *texture,
+void shell_draw_clock (StDrawingArea       *area,
 	               int                  hour,
 	               int                  minute);
 
