@@ -21,6 +21,20 @@
  *
  */
 
+/* Portions copied from Clutter:
+ * Clutter.
+ *
+ * An OpenGL based 'interactive canvas' library.
+ *
+ * Authored By Matthew Allum  <mallum@openedhand.com>
+ *
+ * Copyright (C) 2006 OpenedHand
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ */
+
 /**
  * SECTION:st-box-layout
  * @short_description: a layout container arranging children in a single line
@@ -97,6 +111,8 @@ scrollable_set_adjustments (StScrollable *scrollable,
 {
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (scrollable)->priv;
 
+  g_object_freeze_notify (G_OBJECT (scrollable));
+
   if (hadjustment != priv->hadjustment)
     {
       if (priv->hadjustment)
@@ -116,6 +132,7 @@ scrollable_set_adjustments (StScrollable *scrollable,
         }
 
       priv->hadjustment = hadjustment;
+      g_object_notify (G_OBJECT (scrollable), "hadjustment");
     }
 
   if (vadjustment != priv->vadjustment)
@@ -137,7 +154,10 @@ scrollable_set_adjustments (StScrollable *scrollable,
         }
 
       priv->vadjustment = vadjustment;
+      g_object_notify (G_OBJECT (scrollable), "vadjustment");
     }
+
+  g_object_thaw_notify (G_OBJECT (scrollable));
 }
 
 static void
@@ -146,82 +166,14 @@ scrollable_get_adjustments (StScrollable  *scrollable,
                             StAdjustment **vadjustment)
 {
   StBoxLayoutPrivate *priv;
-  ClutterActor *actor, *stage;
 
   priv = (ST_BOX_LAYOUT (scrollable))->priv;
 
-  actor = CLUTTER_ACTOR (scrollable);
-  stage = clutter_actor_get_stage (actor);
-
-  if (hadjustment)
-    {
-      if (priv->hadjustment)
-        *hadjustment = priv->hadjustment;
-      else
-        {
-          StAdjustment *adjustment;
-          gdouble width, stage_width, increment;
-
-          if (stage)
-            {
-              width = clutter_actor_get_width (actor);
-              stage_width = clutter_actor_get_width (stage);
-              increment = MAX (1.0, MIN (stage_width, width));
-            }
-          else
-            {
-              width = increment = 1.0;
-            }
-
-          adjustment = st_adjustment_new (0,
-                                          0,
-                                          width,
-                                          1.0,
-                                          increment,
-                                          increment);
-
-          scrollable_set_adjustments (scrollable,
-                                      adjustment,
-                                      priv->vadjustment);
-
-          *hadjustment = adjustment;
-        }
-    }
+  if (priv->hadjustment)
+    *hadjustment = priv->hadjustment;
 
   if (vadjustment)
-    {
-      if (priv->vadjustment)
-        *vadjustment = priv->vadjustment;
-      else
-        {
-          StAdjustment *adjustment;
-          gdouble height, stage_height, increment;
-
-          if (stage)
-            {
-              height = clutter_actor_get_height (actor);
-              stage_height = clutter_actor_get_height (stage);
-              increment = MAX (1.0, MIN (stage_height, height));
-            }
-          else
-            {
-              height = increment = 1.0;
-            }
-
-          adjustment = st_adjustment_new (0,
-                                          0,
-                                          height,
-                                          1.0,
-                                          increment,
-                                          increment);
-
-          scrollable_set_adjustments (scrollable,
-                                      priv->hadjustment,
-                                      adjustment);
-
-          *vadjustment = adjustment;
-        }
-    }
+    *vadjustment = priv->vadjustment;
 }
 
 
@@ -242,11 +194,7 @@ st_box_container_add_actor (ClutterContainer *container,
 {
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
 
-  clutter_actor_set_parent (actor, CLUTTER_ACTOR (container));
-
-  priv->children = g_list_append (priv->children, actor);
-
-  g_signal_emit_by_name (container, "actor-added", actor);
+  _st_container_add_actor (container, actor, &priv->children);
 }
 
 static void
@@ -255,28 +203,7 @@ st_box_container_remove_actor (ClutterContainer *container,
 {
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
 
-  GList *item = NULL;
-
-  item = g_list_find (priv->children, actor);
-
-  if (item == NULL)
-    {
-      g_warning ("Actor of type '%s' is not a child of container of type '%s'",
-                 g_type_name (G_OBJECT_TYPE (actor)),
-                 g_type_name (G_OBJECT_TYPE (container)));
-      return;
-    }
-
-  g_object_ref (actor);
-
-  priv->children = g_list_delete_link (priv->children, item);
-  clutter_actor_unparent (actor);
-
-  g_signal_emit_by_name (container, "actor-removed", actor);
-
-  g_object_unref (actor);
-
-  clutter_actor_queue_relayout ((ClutterActor*) container);
+  _st_container_remove_actor (container, actor, &priv->children);
 }
 
 static void
@@ -286,7 +213,8 @@ st_box_container_foreach (ClutterContainer *container,
 {
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
 
-  g_list_foreach (priv->children, (GFunc) callback, callback_data);
+  _st_container_foreach (container, callback, callback_data,
+                         &priv->children);
 }
 
 static void
@@ -294,8 +222,9 @@ st_box_container_lower (ClutterContainer *container,
                         ClutterActor     *actor,
                         ClutterActor     *sibling)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
+
+  _st_container_lower (container, actor, sibling, &priv->children);
 }
 
 static void
@@ -303,15 +232,17 @@ st_box_container_raise (ClutterContainer *container,
                         ClutterActor     *actor,
                         ClutterActor     *sibling)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
+
+  _st_container_raise (container, actor, sibling, &priv->children);
 }
 
 static void
 st_box_container_sort_depth_order (ClutterContainer *container)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
+
+  _st_container_sort_depth_order (container, &priv->children);
 }
 
 static void
@@ -403,11 +334,7 @@ st_box_layout_dispose (GObject *object)
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (object)->priv;
 
   while (priv->children)
-    {
-      clutter_actor_unparent (CLUTTER_ACTOR (priv->children->data));
-
-      priv->children = g_list_delete_link (priv->children, priv->children);
-    }
+    clutter_actor_destroy (priv->children->data);
 
   if (priv->hadjustment)
     {
@@ -432,6 +359,7 @@ get_content_preferred_width (StBoxLayout *self,
 {
   StBoxLayoutPrivate *priv = self->priv;
   gint n_children = 0;
+  gint n_fixed = 0;
   gfloat min_width, natural_width;
   GList *l;
 
@@ -440,34 +368,44 @@ get_content_preferred_width (StBoxLayout *self,
 
   for (l = priv->children; l; l = g_list_next (l))
     {
+      ClutterActor *child = l->data;
       gfloat child_min = 0, child_nat = 0;
+      gboolean child_fill;
 
-      if (!CLUTTER_ACTOR_IS_VISIBLE ((ClutterActor*) l->data))
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
         continue;
 
       n_children++;
 
-      clutter_actor_get_preferred_width ((ClutterActor*) l->data,
-                                         (!priv->is_vertical) ? for_height : -1,
-                                         &child_min,
-                                         &child_nat);
+      if (clutter_actor_get_fixed_position_set (child))
+        {
+          n_fixed++;
+          continue;
+        }
 
       if (priv->is_vertical)
         {
+          _st_actor_get_preferred_width (child, -1, FALSE,
+                                         &child_min, &child_nat);
           min_width = MAX (child_min, min_width);
           natural_width = MAX (child_nat, natural_width);
         }
       else
         {
+          clutter_container_child_get ((ClutterContainer*) self, child,
+                                       "y-fill", &child_fill,
+                                       NULL);
+          _st_actor_get_preferred_width (child, for_height, child_fill,
+                                         &child_min, &child_nat);
           min_width += child_min;
           natural_width += child_nat;
         }
     }
 
-  if (!priv->is_vertical && n_children > 1)
+  if (!priv->is_vertical && (n_children - n_fixed) > 1)
     {
-      min_width += priv->spacing * (n_children - 1);
-      natural_width += priv->spacing * (n_children - 1);
+      min_width += priv->spacing * (n_children - n_fixed - 1);
+      natural_width += priv->spacing * (n_children - n_fixed - 1);
     }
 
   if (min_width_p)
@@ -502,6 +440,7 @@ get_content_preferred_height (StBoxLayout *self,
 {
   StBoxLayoutPrivate *priv = self->priv;
   gint n_children = 0;
+  gint n_fixed = 0;
   gfloat min_height, natural_height;
   GList *l;
 
@@ -510,17 +449,32 @@ get_content_preferred_height (StBoxLayout *self,
 
   for (l = priv->children; l; l = g_list_next (l))
     {
+      ClutterActor *child = l->data;
       gfloat child_min = 0, child_nat = 0;
+      gboolean child_fill = FALSE;
 
-      if (!CLUTTER_ACTOR_IS_VISIBLE ((ClutterActor*) l->data))
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
         continue;
 
       n_children++;
 
-      clutter_actor_get_preferred_height ((ClutterActor*) l->data,
-                                          (priv->is_vertical) ? for_width : -1,
-                                          &child_min,
-                                          &child_nat);
+      if (clutter_actor_get_fixed_position_set (child))
+        {
+          n_fixed++;
+          continue;
+        }
+
+      if (priv->is_vertical)
+        {
+          clutter_container_child_get ((ClutterContainer*) self, child,
+                                       "x-fill", &child_fill,
+                                       NULL);
+        }
+      _st_actor_get_preferred_height (child,
+                                      (priv->is_vertical) ? for_width : -1,
+                                      child_fill,
+                                      &child_min,
+                                      &child_nat);
 
       if (!priv->is_vertical)
         {
@@ -534,10 +488,10 @@ get_content_preferred_height (StBoxLayout *self,
         }
     }
 
-  if (priv->is_vertical && n_children > 1)
+  if (priv->is_vertical && (n_children - n_fixed) > 1)
     {
-      min_height += priv->spacing * (n_children - 1);
-      natural_height += priv->spacing * (n_children - 1);
+      min_height += priv->spacing * (n_children - n_fixed - 1);
+      natural_height += priv->spacing * (n_children - n_fixed - 1);
     }
 
   if (min_height_p)
@@ -553,11 +507,25 @@ st_box_layout_get_preferred_height (ClutterActor *actor,
                                     gfloat       *min_height_p,
                                     gfloat       *natural_height_p)
 {
+  StBoxLayout *self = ST_BOX_LAYOUT (actor);
+  StBoxLayoutPrivate *priv = self->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
 
   st_theme_node_adjust_for_width (theme_node, &for_width);
 
-  get_content_preferred_height (ST_BOX_LAYOUT (actor), for_width,
+  if (priv->hadjustment)
+    {
+      /* If we're scrolled, the parent calls us with the width that
+       * we'll actually get, which can be smaller than the minimum
+       * width that we give our contents.
+       */
+      gfloat min_width;
+
+      get_content_preferred_width (self, -1, &min_width, NULL);
+      for_width = MAX (for_width, min_width);
+    }
+
+  get_content_preferred_height (self, for_width,
                                 min_height_p, natural_height_p);
 
   st_theme_node_adjust_preferred_height (theme_node,
@@ -607,25 +575,41 @@ compute_shrinks (StBoxLayout *self,
    */
 
   /* Find the amount of possible shrink for each child */
-  int n_visible_children = 0;
+  int n_possible_shrink_children = 0;
   for (l = priv->children, i = 0; l; l = l->next, i++)
     {
+      ClutterActor *child = l->data;
       gfloat child_min, child_nat;
+      gboolean child_fill;
+      gboolean fixed;
+
+      child = (ClutterActor*) l->data;
+      fixed = clutter_actor_get_fixed_position_set (child);
 
       shrinks[i].child_index = i;
-      if (CLUTTER_ACTOR_IS_VISIBLE (l->data))
+      if (CLUTTER_ACTOR_IS_VISIBLE (child) && !fixed)
         {
           if (priv->is_vertical)
-            clutter_actor_get_preferred_height ((ClutterActor*) l->data,
-                                                for_length,
-                                                &child_min, &child_nat);
+            {
+              clutter_container_child_get ((ClutterContainer*) self, child,
+                                           "x-fill", &child_fill,
+                                           NULL);
+              _st_actor_get_preferred_height (child,
+                                              for_length, child_fill,
+                                              &child_min, &child_nat);
+            }
           else
-            clutter_actor_get_preferred_width ((ClutterActor*) l->data,
-                                               for_length,
-                                               &child_min, &child_nat);
+            {
+              clutter_container_child_get ((ClutterContainer*) self, child,
+                                           "y-fill", &child_fill,
+                                           NULL);
+              _st_actor_get_preferred_width (child,
+                                             for_length, child_fill,
+                                             &child_min, &child_nat);
+            }
 
           shrinks[i].shrink_amount = MAX (0., child_nat - child_min);
-          n_visible_children++;
+          n_possible_shrink_children++;
         }
       else
         {
@@ -654,15 +638,15 @@ compute_shrinks (StBoxLayout *self,
 
   /* Start by moving the line downward, top-of-bar by top-of-bar */
   shrink_so_far = 0;
-  for (n_shrink_children = 1; n_shrink_children <= n_visible_children; n_shrink_children++)
+  for (n_shrink_children = 1; n_shrink_children <= n_possible_shrink_children; n_shrink_children++)
     {
-      if (n_shrink_children < n_visible_children)
+      if (n_shrink_children < n_possible_shrink_children)
         base_shrink = shrinks[n_shrink_children].shrink_amount;
       else
         base_shrink = 0;
       shrink_so_far += n_shrink_children * (shrinks[n_shrink_children - 1].shrink_amount - base_shrink);
 
-      if (shrink_so_far >= total_shrink || n_shrink_children == n_visible_children)
+      if (shrink_so_far >= total_shrink || n_shrink_children == n_possible_shrink_children)
         break;
     }
 
@@ -697,6 +681,8 @@ st_box_layout_allocate (ClutterActor          *actor,
   gint n_expand_children = 0, i;
   gfloat expand_amount, shrink_amount;
   BoxChildShrink *shrinks = NULL;
+  gboolean flip = (st_widget_get_direction (ST_WIDGET (actor)) == ST_TEXT_DIRECTION_RTL)
+                   && (!priv->is_vertical);
 
   CLUTTER_ACTOR_CLASS (st_box_layout_parent_class)->allocate (actor, box,
                                                               flags);
@@ -709,10 +695,11 @@ st_box_layout_allocate (ClutterActor          *actor,
   avail_width  = content_box.x2 - content_box.x1;
   avail_height = content_box.y2 - content_box.y1;
 
-  get_content_preferred_height (ST_BOX_LAYOUT (actor), avail_width,
-                                &min_height, &natural_height);
   get_content_preferred_width (ST_BOX_LAYOUT (actor), avail_height,
                                &min_width, &natural_width);
+  get_content_preferred_height (ST_BOX_LAYOUT (actor), MAX (avail_width, min_width),
+                                &min_height, &natural_height);
+
 
   /* update adjustments for scrolling */
   if (priv->vadjustment)
@@ -721,10 +708,10 @@ st_box_layout_allocate (ClutterActor          *actor,
 
       g_object_set (G_OBJECT (priv->vadjustment),
                     "lower", 0.0,
-                    "upper", natural_height,
+                    "upper", MAX (min_height, avail_height),
                     "page-size", avail_height,
                     "step-increment", avail_height / 6,
-                    "page-increment", avail_height,
+                    "page-increment", avail_height - avail_height / 6,
                     NULL);
 
       prev_value = st_adjustment_get_value (priv->vadjustment);
@@ -737,25 +724,37 @@ st_box_layout_allocate (ClutterActor          *actor,
 
       g_object_set (G_OBJECT (priv->hadjustment),
                     "lower", 0.0,
-                    "upper", natural_width,
+                    "upper", MAX (min_width, avail_width),
                     "page-size", avail_width,
                     "step-increment", avail_width / 6,
-                    "page-increment", avail_width,
+                    "page-increment", avail_width - avail_width / 6,
                     NULL);
 
       prev_value = st_adjustment_get_value (priv->hadjustment);
       st_adjustment_set_value (priv->hadjustment, prev_value);
     }
 
+  if (avail_height < min_height)
+    {
+      avail_height = min_height;
+      content_box.y2 = content_box.y1 + avail_height;
+    }
+
+  if (avail_width < min_width)
+    {
+      avail_width = min_width;
+      content_box.x2 = content_box.x1 + avail_width;
+    }
+
   if (priv->is_vertical)
     {
       expand_amount = MAX (0, avail_height - natural_height);
-      shrink_amount = MAX (0, natural_height - MAX (avail_height, min_height));
+      shrink_amount = MAX (0, natural_height - avail_height);
     }
   else
     {
       expand_amount = MAX (0, avail_width - natural_width);
-      shrink_amount = MAX (0, natural_width - MAX (avail_width, min_width));
+      shrink_amount = MAX (0, natural_width - avail_width);
     }
 
   if (expand_amount > 0)
@@ -764,13 +763,15 @@ st_box_layout_allocate (ClutterActor          *actor,
       n_expand_children = 0;
       for (l = priv->children; l; l = l->next)
         {
+          ClutterActor *child = l->data;
           gboolean expand;
 
-          if (!CLUTTER_ACTOR_IS_VISIBLE (l->data))
+          if (!CLUTTER_ACTOR_IS_VISIBLE (child) ||
+              clutter_actor_get_fixed_position_set (child))
             continue;
 
           clutter_container_child_get ((ClutterContainer *) actor,
-                                       (ClutterActor*) l->data,
+                                       child,
                                        "expand", &expand,
                                        NULL);
           if (expand)
@@ -789,6 +790,8 @@ st_box_layout_allocate (ClutterActor          *actor,
 
   if (priv->is_vertical)
     position = content_box.y1;
+  else if (flip)
+    position = content_box.x2;
   else
     position = content_box.x1;
 
@@ -808,12 +811,19 @@ st_box_layout_allocate (ClutterActor          *actor,
       ClutterActor *child = (ClutterActor*) l->data;
       ClutterActorBox child_box;
       gfloat child_min, child_nat, child_allocated;
-      gboolean xfill, yfill, expand;
+      gboolean xfill, yfill, expand, fixed;
       StAlign xalign, yalign;
 
       if (!CLUTTER_ACTOR_IS_VISIBLE (child))
         goto next_child;
 
+      fixed = clutter_actor_get_fixed_position_set (child);
+      if (fixed)
+        {
+          clutter_actor_allocate_preferred_size (child, flags);
+          goto next_child;
+        }
+
       clutter_container_child_get ((ClutterContainer*) actor, child,
                                    "x-fill", &xfill,
                                    "y-fill", &yfill,
@@ -824,13 +834,13 @@ st_box_layout_allocate (ClutterActor          *actor,
 
       if (priv->is_vertical)
         {
-          clutter_actor_get_preferred_height (child, avail_width,
-                                              &child_min, &child_nat);
+          _st_actor_get_preferred_height (child, avail_width, xfill,
+                                          &child_min, &child_nat);
         }
       else
         {
-          clutter_actor_get_preferred_width (child, avail_height,
-                                             &child_min, &child_nat);
+          _st_actor_get_preferred_width (child, avail_height, yfill,
+                                         &child_min, &child_nat);
         }
 
       child_allocated = child_nat;
@@ -839,37 +849,48 @@ st_box_layout_allocate (ClutterActor          *actor,
       else if (shrink_amount > 0)
         child_allocated -= shrinks[i].shrink_amount;
 
-      next_position = position + child_allocated;
+      if (flip)
+        next_position = position - child_allocated;
+      else
+        next_position = position + child_allocated;
 
       if (priv->is_vertical)
         {
           child_box.y1 = (int)(0.5 + position);
           child_box.y2 = (int)(0.5 + next_position);
           child_box.x1 = content_box.x1;
-          if (priv->hadjustment)
-            child_box.x2 = content_box.x1 + MAX (avail_width, natural_width);
-          else
-            child_box.x2 = content_box.x2;
+          child_box.x2 = content_box.x2;
 
-          _st_allocate_fill (child, &child_box, xalign, yalign, xfill, yfill);
+          _st_allocate_fill (ST_WIDGET (actor), child, &child_box,
+                             xalign, yalign, xfill, yfill);
           clutter_actor_allocate (child, &child_box, flags);
 
         }
       else
         {
-          child_box.x1 = (int)(0.5 + position);
-          child_box.x2 = (int)(0.5 + next_position);
-          child_box.y1 = content_box.y1;
-          if (priv->vadjustment)
-            child_box.y2 = content_box.y1 + MAX (avail_height, natural_height);
+          if (flip)
+            {
+              child_box.x1 = (int)(0.5 + next_position);
+              child_box.x2 = (int)(0.5 + position);
+            }
           else
-            child_box.y2 = content_box.y2;
+            {
+              child_box.x1 = (int)(0.5 + position);
+              child_box.x2 = (int)(0.5 + next_position);
+            }
 
-          _st_allocate_fill (child, &child_box, xalign, yalign, xfill, yfill);
+          child_box.y1 = content_box.y1;
+          child_box.y2 = content_box.y2;
+
+          _st_allocate_fill (ST_WIDGET (actor), child, &child_box,
+                             xalign, yalign, xfill, yfill);
           clutter_actor_allocate (child, &child_box, flags);
         }
 
-      position = next_position + priv->spacing;
+      if (flip)
+        position = next_position - priv->spacing;
+      else
+        position = next_position + priv->spacing;
 
     next_child:
       if (priv->is_pack_start)
@@ -962,10 +983,10 @@ st_box_layout_paint (ClutterActor *actor)
    * the borders and background stay in place; after drawing the borders and
    * background, we clip to the content area */
   if (priv->hadjustment || priv->vadjustment)
-    cogl_clip_push ((int)content_box.x1,
-                    (int)content_box.y1,
-                    (int)content_box.x2 - (int)content_box.x1,
-                    (int)content_box.y2 - (int)content_box.y1);
+    cogl_clip_push_rectangle ((int)content_box.x1,
+                              (int)content_box.y1,
+                              (int)content_box.x2,
+                              (int)content_box.y2);
 
   for (l = priv->children; l; l = g_list_next (l))
     {
@@ -1036,10 +1057,10 @@ st_box_layout_pick (ClutterActor       *actor,
   content_box.y2 += y;
 
   if (priv->hadjustment || priv->vadjustment)
-    cogl_clip_push ((int)content_box.x1,
-                    (int)content_box.y1,
-                    (int)content_box.x2 - (int)content_box.x1,
-                    (int)content_box.y2 - (int)content_box.y1);
+    cogl_clip_push_rectangle ((int)content_box.x1,
+                              (int)content_box.y1,
+                              (int)content_box.x2,
+                              (int)content_box.y2);
 
   for (l = priv->children; l; l = g_list_next (l))
     {
@@ -1225,3 +1246,95 @@ st_box_layout_get_pack_start (StBoxLayout *box)
 
   return box->priv->is_pack_start;
 }
+
+static void
+st_box_layout_internal_remove_all (StBoxLayout *self,
+                                   gboolean     destroy)
+{
+  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (self)->priv;
+  ClutterActor *child;
+
+  while (priv->children)
+    {
+      child = priv->children->data;
+
+      g_object_ref (child);
+      priv->children = g_list_delete_link (priv->children, priv->children);
+      clutter_actor_unparent (child);
+      g_signal_emit_by_name (self, "actor-removed", child);
+      if (destroy)
+        clutter_actor_destroy (child);
+      g_object_unref (child);
+    }
+
+  clutter_actor_queue_relayout ((ClutterActor*) self);
+}
+
+/**
+ * st_box_layout_remove_all:
+ * @self:
+ *
+ * Efficiently unparent all children currently in this box.
+ */
+void
+st_box_layout_remove_all (StBoxLayout *self)
+{
+  st_box_layout_internal_remove_all (self, FALSE);
+}
+
+/**
+ * st_box_layout_destroy_children:
+ * @self:
+ *
+ * Efficiently unparent and destroy all children currently in this box.
+ */
+void
+st_box_layout_destroy_children (StBoxLayout *self)
+{
+  st_box_layout_internal_remove_all (self, TRUE);
+}
+
+/**
+ * st_box_layout_get_n_children:
+ * @self: a #StBoxLayout
+ *
+ * Returns the number of children in this box.
+ */
+guint
+st_box_layout_get_n_children  (StBoxLayout *self)
+{
+  return g_list_length (self->priv->children);
+}
+
+void
+st_box_layout_move_child (StBoxLayout  *self,
+                          ClutterActor *actor,
+                          int           pos)
+{
+  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (self)->priv;
+
+  GList *item = NULL;
+
+  item = g_list_find (priv->children, actor);
+
+  if (item == NULL)
+    {
+      g_warning ("Actor of type '%s' is not a child of the StBoxLayout container",
+                 g_type_name (G_OBJECT_TYPE (actor)));
+      return;
+    }
+
+  priv->children = g_list_delete_link (priv->children, item);
+  priv->children = g_list_insert (priv->children, actor, pos);
+  clutter_actor_queue_relayout ((ClutterActor*) self);
+}
+
+void
+st_box_layout_insert_actor (StBoxLayout  *self,
+                            ClutterActor *actor,
+                            int           pos)
+{
+  clutter_container_add_actor((ClutterContainer*) self, actor);
+  st_box_layout_move_child(self, actor, pos);
+}
+
