@@ -118,9 +118,9 @@ void st_theme_node_get_background_gradient (StThemeNode   *node,
 
 const char *st_theme_node_get_background_image (StThemeNode *node);
 
-double st_theme_node_get_border_width  (StThemeNode  *node,
+int    st_theme_node_get_border_width  (StThemeNode  *node,
                                         StSide        side);
-double st_theme_node_get_border_radius (StThemeNode  *node,
+int    st_theme_node_get_border_radius (StThemeNode  *node,
                                         StCorner      corner);
 void   st_theme_node_get_border_color  (StThemeNode  *node,
                                         StSide        side,
@@ -129,6 +129,9 @@ void   st_theme_node_get_border_color  (StThemeNode  *node,
 double st_theme_node_get_padding       (StThemeNode  *node,
                                         StSide        side);
 
+double st_theme_node_get_horizontal_padding (StThemeNode *node);
+double st_theme_node_get_vertical_padding   (StThemeNode *node);
+
 int    st_theme_node_get_width         (StThemeNode  *node);
 int    st_theme_node_get_height        (StThemeNode  *node);
 int    st_theme_node_get_min_width     (StThemeNode  *node);
