@@ -242,6 +242,12 @@ st_widget_dispose (GObject *gobject)
       priv->background_image_shadow = NULL;
     }
 
+  if (priv->theme_node)
+    {
+      g_object_unref (priv->theme_node);
+      priv->theme_node = NULL;
+    }
+
   if (priv->tooltip)
     {
       ClutterContainer *parent;
@@ -537,9 +543,8 @@ static void
 st_widget_paint (ClutterActor *self)
 {
   StWidgetPrivate *priv = ST_WIDGET (self)->priv;
-  StWidgetClass *klass = ST_WIDGET_GET_CLASS (self);
 
-  klass->draw_background (ST_WIDGET (self));
+  st_widget_real_draw_background (ST_WIDGET (self));
 
   if (priv->background_image != NULL)
     {
@@ -905,8 +910,8 @@ st_widget_real_style_changed (StWidget *self)
       /* `border-image' takes precedence over `background-image'.
        * Firefox lets the background-image shine thru when border-image has
        * alpha an channel, maybe that would be an option for the future. */
-      texture = st_texture_cache_get_texture (texture_cache,
-                                                filename);
+      texture = (ClutterTexture*) st_texture_cache_load_file_simple (texture_cache,
+                                                                     filename);
 
       clutter_texture_get_base_size (CLUTTER_TEXTURE (texture),
                                      &width, &height);
@@ -968,16 +973,8 @@ st_widget_real_style_changed (StWidget *self)
   bg_file = st_theme_node_get_background_image (theme_node);
   if (bg_file != NULL)
     {
-      texture = st_texture_cache_get_texture (texture_cache, bg_file);
-      priv->background_image = (ClutterActor*) texture;
-
-      if (priv->background_image != NULL)
-        {
-          clutter_actor_set_parent (priv->background_image,
-                                    CLUTTER_ACTOR (self));
-        }
-      else
-        g_warning ("Could not load %s", bg_file);
+      priv->background_image = st_texture_cache_load_file_simple (texture_cache, bg_file);
+      clutter_actor_set_parent (priv->background_image, CLUTTER_ACTOR (self));
 
       has_changed = TRUE;
       relayout_needed = TRUE;
@@ -1110,8 +1107,7 @@ st_widget_get_theme_node (StWidget *widget)
 
       if (stage == NULL)
         {
-          g_warning ("st_widget_get_theme_node called on a widget not in a stage");
-          stage = CLUTTER_STAGE (clutter_stage_get_default ());
+          g_error ("st_widget_get_theme_node called on a widget not in a stage");
         }
 
       if (parent_node == NULL)
@@ -1198,7 +1194,6 @@ st_widget_class_init (StWidgetClass *klass)
   actor_class->leave_event = st_widget_leave;
   actor_class->hide = st_widget_hide;
 
-  klass->draw_background = st_widget_real_draw_background;
   klass->style_changed = st_widget_real_style_changed;
 
   /**
@@ -1581,40 +1576,6 @@ st_widget_set_direction (StWidget *self, StTextDirection dir)
   self->priv->direction = dir;
 }
 
-/**
- * st_widget_get_border_image:
- * @actor: A #StWidget
- *
- * Get the texture used as the border image. This is set using the
- * "border-image" CSS property. This function should normally only be used
- * by subclasses.
- *
- * Returns: (transfer none): #ClutterActor
- */
-ClutterActor *
-st_widget_get_border_image (StWidget *actor)
-{
-  StWidgetPrivate *priv = ST_WIDGET (actor)->priv;
-  return priv->border_image;
-}
-
-/**
- * st_widget_get_background_image:
- * @actor: A #StWidget
- *
- * Get the texture used as the background image. This is set using the
- * "background-image" CSS property. This function should normally only be used
- * by subclasses.
- *
- * Returns: (transfer none): a #ClutterActor
- */
-ClutterActor *
-st_widget_get_background_image (StWidget *actor)
-{
-  StWidgetPrivate *priv = ST_WIDGET (actor)->priv;
-  return priv->background_image;
-}
-
 /**
  * st_widget_set_has_tooltip:
  * @widget: A #StWidget
@@ -1776,27 +1737,3 @@ st_widget_hide_tooltip (StWidget *widget)
   if (widget->priv->tooltip)
     st_tooltip_hide (widget->priv->tooltip);
 }
-
-/**
- * st_widget_draw_background:
- * @widget: a #StWidget
- *
- * Invokes #StWidget::draw_background() using the default background
- * image and/or color from the @widget style
- *
- * This function should be used by subclasses of #StWidget that override
- * the paint() virtual function and cannot chain up
- */
-void
-st_widget_draw_background (StWidget *self)
-{
-  StWidgetPrivate *priv;
-  StWidgetClass *klass;
-
-  g_return_if_fail (ST_IS_WIDGET (self));
-
-  priv = self->priv;
-
-  klass = ST_WIDGET_GET_CLASS (self);
-  klass->draw_background (ST_WIDGET (self));
-}
